package com.example.produceItemService.service;


import com.example.produceItemService.entity.Item;
import com.example.produceItemService.repository.ItemRepository;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.reactive.function.client.WebClient;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@Service
@Slf4j
@RequiredArgsConstructor
@Transactional
public class ItemService {

    @Autowired
    private ItemRepository itemRepository;

    private WebClient webClient = WebClient.create();

    public Mono<Item> createItem(Item Item){
        return itemRepository.save(Item);
    }

    public Flux<Item> getAllItems(){
        return itemRepository.findAll();
    }

    public Mono<Item> findById(Integer ItemId){
        return itemRepository.findById(ItemId);
    }

    public Mono<Item> updateItem(Integer ItemId,  Item Item){
        return itemRepository.findById(ItemId)
                .flatMap(dbItem -> {
                    dbItem.setQuantity(Item.getQuantity());
                    dbItem.setPrice(Item.getPrice());
                    return itemRepository.save(dbItem);
                });
    }

    public Mono<Item> deleteItem(Integer ItemId){
        return itemRepository.findById(ItemId)
                .flatMap(existingItem -> itemRepository.delete(existingItem)
                        .then(Mono.just(existingItem)));
    }

    //TODO config timeout of read and write timeout for handler

    public Mono<Item> produceItem(Item item) {
        try {
        ObjectMapper objectMapper = new ObjectMapper();
        String body = objectMapper.writeValueAsString(item);
        return webClient.post()
                    .uri("http://localhost:9090/items/receive")
                    .contentType(MediaType.APPLICATION_JSON)
                    .accept(MediaType.APPLICATION_JSON)
                    .bodyValue(body)
                    .retrieve()
                    .bodyToMono(Item.class);
        } catch (JsonProcessingException e) {
            throw new RuntimeException(e);
        }
    }

//    public Mono<Item> produceItem(Integer itemId) {
//        try {
//        ObjectMapper objectMapper = new ObjectMapper();
//        String body = objectMapper.writeValueAsString(item);
//        return webClient.post()
//                    .uri("http://localhost:9090/items/receive")
//                    .contentType(MediaType.APPLICATION_JSON)
//                    .accept(MediaType.APPLICATION_JSON)
//                    .bodyValue(body)
//                    .retrieve()
//                    .bodyToMono(Item.class);
//        } catch (JsonProcessingException e) {
//            throw new RuntimeException(e);
//        }
//    }
}
